#!/bin/sh

export EASYRSA=/etc/openvpn/easy-rsa

newclient () {
	# Build client certificate
	cd $EASYRSA
	EASYRSA_CERT_EXPIRE=3650 easyrsa build-client-full $1 nopass
	if [ "$?" != "0" ]; then
      echo "Error, could not create the client certificate for profile!"
      exit -3
	fi
	# Generates the custom client.ovpn
	cp /etc/openvpn/client-common.txt /etc/openvpn/clientprofiles/$1.ovpn
	echo "<ca>" >> /etc/openvpn/clientprofiles/$1.ovpn
	cat $EASYRSA/pki/ca.crt >> /etc/openvpn/clientprofiles/$1.ovpn
	echo "</ca>" >> /etc/openvpn/clientprofiles/$1.ovpn
	echo "<cert>" >> /etc/openvpn/clientprofiles/$1.ovpn
	sed -ne '/BEGIN CERTIFICATE/,$ p' $EASYRSA/pki/issued/$1.crt >> /etc/openvpn/clientprofiles/$1.ovpn
	echo "</cert>" >> /etc/openvpn/clientprofiles/$1.ovpn
	echo "<key>" >> /etc/openvpn/clientprofiles/$1.ovpn
	cat $EASYRSA/pki/private/$1.key >> /etc/openvpn/clientprofiles/$1.ovpn
	echo "</key>" >> /etc/openvpn/clientprofiles/$1.ovpn
	echo "<tls-auth>" >> /etc/openvpn/clientprofiles/$1.ovpn
	sed -ne '/BEGIN OpenVPN Static key/,$ p' /etc/openvpn/ta.key >> /etc/openvpn/clientprofiles/$1.ovpn
	echo "</tls-auth>" >> /etc/openvpn/clientprofiles/$1.ovpn
	# Done
	echo
	echo "Client $1 added, configuration is available at:" /etc/openvpn/clientprofiles/"$1.ovpn"
	exit 0
}

if [ -d /etc/openvpn/easy-rsa ]; then
  if [ "$1" != "" ]; then
    newclient $1
    if [ "$?" != 0 ]; then
      echo "Error while trying to create the profile named $1."
      exit -3
    fi
    echo "Added profile $1."
    exit 0
  else
    echo "No profile name given!"
    exit -2
  fi
else
  echo "OpenVPN not configured!"
  exit -1
fi
