/*
 * ##################################################### Start MainSite #####################################################
 */
function MainSite() {
  if (mainSite) {
    return mainSite;
  }
  if (!(this instanceof MainSite)) {
    return new MainSite();
  }
  var thisClass = this;
  
  this.updateProcess = undefined;
  // UI variables
  // Do not change defaults in the Sites, use the optional configureManufacturerSettings/configureProductSettings
  // prototype function mechanism via Manufacturer.js / Product.js
  this.showHeaderMenu = false;
  this.showFooterMenu = false;
  this.showSystemMenu = true;
  this.disclaimerMenuItem = true;
  this.licenseMenuItem = true;
  
  this.companyUrl = "https://www.ise.de";
  this.companyName = "ise Individuelle Software und Elektronik GmbH";
  this.kimFriendlyName = 'Gira S1';
  
  this.logoColumn = "1";  
  this.logoWidth = "102px";
  this.logoHeight = "58px";
  this.logoAltText = "Logo ise GmbH";
  
  this.showExpandPageIcon = false;
  this.logoColumnSize = "9";
  this.navColumnSize = "9";
  this.withHeaderSeparator = false;
  
  this.knxEnabled = true;
  
  // variables
  this.requestsEnabled = true;
  this.rebootBecauseOfAnUpdate = false;
  this.connectionError = false;
  this.homepage = undefined;
  this.pages = {};
  this.deviceInfo = undefined;
  this.shortDeviceInfo = undefined;
  this.currentCOValues = undefined;
  this.appState = undefined;
  this.dataBindings = [];
  this.menuElement = [];

  this.appFeatures = {};
  
  // now check for manufacturer/product modifications.
  if (typeof thisClass.configureManufacturerSettings === 'function') {
    thisClass.configureManufacturerSettings();
  }

  if (typeof thisClass.configureProductSettings === 'function') {
    thisClass.configureProductSettings();
  }
  
  this.setAppFeatures = function(app, response){
    if (response && response.data != undefined){
      thisClass.appFeatures[app] = response.data;
    }
  }
  
  this.getMenuTarget = function(name) {
    for (var prop in this.menu) {
      if (prop && prop != "first" && prop != "last" && this.menu[prop] && this.menu[prop].target) {
        if (this.menu[prop].name == name) {
          return this.menu[prop].target;
        }
        if (typeof(this.menu[prop].target) == "object") {
          for (var pprop in this.menu[prop].target) {
            if (pprop && pprop != "first" && pprop != "last" && this.menu[prop].target[pprop] && this.menu[prop].target[pprop].target) {
              if (this.menu[prop].target[pprop].name == name) {
                return this.menu[prop].target[pprop].target;
              }
            }
          }
        }
      }
    }
  }
  
  this.menu = new Menu(new MenuItem('device_status', 'StatusPage'));

  var menutarget = this.menu; 
  if (this.showSystemMenu) {
    this.menu.add(new MenuItem('system', new Menu(new MenuItem('download_logs', 'DownloadLogs'))));
    menutarget = this.menu.system.target;
  } else {
    menutarget.add(new MenuItem('download_logs', 'DownloadLogs'));
  }
  menutarget.add(new MenuItem('reboot', 'RebootDevice'));
  menutarget.add(new MenuItem('factory_reset', 'FactoryReset'));
  menutarget.add(new MenuItem('firmware_update', 'FirmwareUpdatePage'));
  this.pages['StatusPage'] = new StatusPage();
  this.pages['FirmwareUpdatePage'] = new FirmwareUpdatePage();
  if (this.disclaimerMenuItem) {
    menutarget.add(new MenuItem('disclaimer', 'Disclaimer'));
  }
  if (this.licenseMenuItem) {
    menutarget.add(new MenuItem('licenses', 'LicensesPage'));
  }
  this.pages['LicensesPage'] = new LicensesPage();
  this.topnav = undefined;

  this.headerMenu = undefined;
  this.headerNav = undefined;

  this.footerMenu = undefined;
  this.footerNav = undefined;


  var main = undefined;
  this.modal = undefined;
  this.modalWaitForContent = undefined;
  this.currentModalDialog = undefined;

  this.createNavMenu = function() {
    if (typeof thisClass.addProductTopNavMenuItems === 'function')
      thisClass.addProductTopNavMenuItems();
    var menuItem = this.menu.first;
    while (menuItem) {
      thisClass.addMenuItem(menuItem, thisClass.topnav);
      menuItem = menuItem.next;
    }
    if (thisClass.headerMenu && thisClass.headerNav) {
      var headerMenuItem = thisClass.headerMenu.first;
      while (headerMenuItem) {
        thisClass.addMenuItem(headerMenuItem, thisClass.headerNav);
        headerMenuItem = headerMenuItem.next;
      }
    }
    thisClass.footerMenu = new Menu(new MenuItem('disclaimer', 'Disclaimer'));
    thisClass.footerMenu.add(new MenuItem('licenses', 'LicensesPage'));
    if (thisClass.footerMenu && thisClass.footerNav) {
      var footerMenuItem = thisClass.footerMenu.first;
      while (footerMenuItem) {
        thisClass.addMenuItem(footerMenuItem, thisClass.footerNav);
        footerMenuItem = footerMenuItem.next;
      }
    }
  };

  this.addMenuItem = function(item, parentmenuitem) {
    var listElem = document.createElement('li');
    if (item.target instanceof Menu) {
      var divSubmenu = document.createElement('div');
      divSubmenu.setAttribute('tabindex', '0');
      divSubmenu.setAttribute('class', 'dropdown');
      var ankerElem = document.createElement('a');
      ankerElem.appendChild(document.createTextNode(i18n.translate(item.name)));
      divSubmenu.appendChild(ankerElem);
      var ulSubmenu = document.createElement('ul');
      ulSubmenu.setAttribute('class', 'dropdown-content navdropdown');
      divSubmenu.appendChild(ulSubmenu);
      if (navigator.userAgent.match(/iPad|Phone|Pod/i) != null) {
        ulSubmenu.style.opacity = '1';
        ulSubmenu.style.visibility = 'hidden';
        ulSubmenu.setAttribute('class', 'dropdown-content navdropdown ipaddropdown');
        divSubmenu.ontouchstart = function(evt) {
          var elements = document.getElementsByClassName('ipaddropdown');
          if (ulSubmenu.style.visibility == 'hidden') {
            ulSubmenu.style.visibility = 'visible';
            var hideElement = function(evt) {
              ulSubmenu.style.visibility = 'hidden';
              this.removeEventListener('touchstart', hideElement, false);
            };
            setTimeout(function() {
              document.addEventListener("touchstart", hideElement, false)
            }, 100);;
          }
        };
      }
      var menuItem = item.target.first;
      while (menuItem) {
        this.addMenuItem(menuItem, ulSubmenu);
        menuItem = menuItem.next;
      }
      listElem.appendChild(divSubmenu);
    } else {
      var ankerElem = document.createElement('a');
      ankerElem.setAttribute('href', '#' + item.target);
      ankerElem.onclick = function() {
        return thisClass.show(item.target);
      };
      ankerElem.appendChild(document.createTextNode(i18n.translate(item.name)));
      listElem.appendChild(ankerElem);
      thisClass.menuElement[item.target] = listElem;
    }
    parentmenuitem.appendChild(listElem);
  };

  this.oneShotAllTimer = function() {
    for (var timerName in thisClass.timer) {
      thisClass.timer[timerName].oneShot();
    }
  }
  this.abortAllTimer = function() {
    for (var timerName in thisClass.timer) {
      if (mainSite.timer.getDeviceInfo != thisClass.timer[timerName]){
        thisClass.timer[timerName].abortTimeout();
      }
    }
  }

  this.hideMenuElement = function(menuItemTarget){
    if (thisClass.menuElement[menuItemTarget]){
      thisClass.menuElement[menuItemTarget].style.display = 'none';
    }
  }
  this.showMenuElement = function(menuItemTarget){
    if (thisClass.menuElement[menuItemTarget]){
      thisClass.menuElement[menuItemTarget].style.display = '';
    }
  }

  this.hideTopNavMenu = function(){
    if (thisClass.topnav){
      thisClass.topnav.style.visibility = 'hidden';
    }
  }
  this.showTopNavMenu = function(){
    if (thisClass.topnav){
      thisClass.topnav.style.visibility = '';
    }
  }
  
  this.getParameterStringToList = function(paramterString) {
    if (!paramterString)
      return {};
    var parameters = paramterString.split('&');
    var parameterList = {};
    for (var keyNValue in parameters) {
      var result = parameters[keyNValue].split('=');
      parameterList[result[0]] = decodeURIComponent(result[1]);
    }
    return parameterList;
  }

  this.show = function(nextPageRaw) {
    var pageNParameter = (nextPageRaw != undefined ? nextPageRaw.split('?') : {});
    var nextPage = pageNParameter[0] || nextPageRaw;
    var nextPageParameter = thisClass.getParameterStringToList(pageNParameter[1] || '');
    switch (nextPage) {
      case 'modal':
        return true;
      case undefined:
      case '#close':
      case '#':
      case '':
        for (var pageToHide in thisClass.pages) {
          if (pageToHide != nextPage) {
            thisClass.pages[pageToHide].hide();
          }
        }
        location.hash = '';
        thisClass.homepage.parameter = nextPageParameter;
        thisClass.homepage.show();
        document.body.appendChild(mainSite.modalWaitForContent);
        return true;
      case thisClass.getMenuTarget("reboot"):
        if (thisClass.pages.LoginPage && thisClass.pages.LoginPage.visible) {
          return false;
        }
        thisClass.modal.setAttribute('id', thisClass.getMenuTarget("reboot"));
        new RebootDevice().prepare();
        return true;
      case thisClass.getMenuTarget("factory_reset"):
        if (thisClass.pages.LoginPage && thisClass.pages.LoginPage.visible) {
          return false;
        }
        thisClass.modal.setAttribute('id', thisClass.getMenuTarget("factory_reset"));
        new FactoryReset().prepare();
        return true;
      case 'Disclaimer':
        new Disclaimer().prepare();
        return true;
      case thisClass.getMenuTarget("download_logs"):
        if (thisClass.pages.LoginPage && thisClass.pages.LoginPage.visible) {
          return false;
        }
        if (thisClass.modal.id == nextPage) {
          return false;
        }
        thisClass.modal.setAttribute('id', thisClass.getMenuTarget("download_logs"));
        new DownloadLogs().prepare();
        return true;
      default:
        for (var pageToHide in thisClass.pages) {
          if (pageToHide != nextPage) {
            thisClass.pages[pageToHide].hide();
          }
        }
        if (thisClass.pages[nextPage] !== undefined) {
          thisClass.pages[nextPage].parameter = nextPageParameter;
          thisClass.pages[nextPage].show();
        } else {
          thisClass.homepage.parameter = nextPageParameter;
          thisClass.homepage.show();
        }
        return true;
    }
    location.hash = '';
    document.body.appendChild(mainSite.modalWaitForContent);
    return false;
  }

  var createLanguageRow = function() {
    // default language English
    var language = "en";
    // try get get a value from the browser (not always senseful)
    var browserLang = navigator.language || navigator.userLanguage;
    language = typeof browserLang !== 'undefined' ? browserLang : "en";
    // Check language cookie and set the language, default is "en"
    var cookieLang = (typeof window.mainSite.localStorage !== 'undefined') ? mainSite.localStorage.getItem("isc_language") : undefined;
    language = cookieLang ? cookieLang : language;

    i18n.userSelected(language);

    // build language selection
    var langSelection = document.createElement('span');
    langSelection.setAttribute('class', 'glyphicons globecolor globe_af');
    langSelection.setAttribute('column', '1');
    var langSelect = document.createElement('select');
    langSelection.appendChild(langSelect);
    langSelect.setAttribute('class', 'langSelection input--small')
    langSelect.setAttribute('column', '1')
    langSelect.onchange = function() {
      // Store as cookie
      document.cookie = "isc_language=" + this.value;
      if (typeof window.mainSite.localStorage !== 'undefined')
        mainSite.localStorage.setItem('isc_language', this.value);
      // and reload page
      location.reload(true);
    };

    // The supported language names and codes are stored in i18n
    for (var i = 0; i < i18n.lang_names.length; i++) {
      var langOption = document.createElement('option');
      langOption.value = i18n.lang_codes[i];
      // cut locale, use only main code, e.g. "de"
      if (language.substring(0, 2) === i18n.lang_codes[i]) {
        langOption.setAttribute('selected', '');
      }
      langOption.appendChild(document.createTextNode(i18n.lang_names[i]));
      langSelect.appendChild(langOption);
    }

    var languageRow = document.createElement('div');
    languageRow.setAttribute('row', '');
    languageRow.setAttribute('class', 'langBlock');
    var p2 = document.createElement('p');
    p2.setAttribute('column', '9');
    p2.appendChild(langSelection);
    languageRow.appendChild(p2);
    return languageRow;
  }

  var createHeaderRow = function() {
    var div = document.createElement('div');
    div.setAttribute('row', '');
    var h1 = document.createElement('h3');
    if (thisClass.logoColumnSize) {
      h1.setAttribute('column', thisClass.logoColumnSize);
    }
    h1.setAttribute('class', 'h3 logo');
    var title = document.createTextNode('');
    h1.appendChild(title);
    thisClass.dataBindings.push(
      function() {
        document.title = (thisClass.shortDeviceInfo) ? thisClass.shortDeviceInfo['KIM-FriendlyName'] : thisClass.kimFriendlyName;
        title.nodeValue = document.title;
      });
    var divLogo = document.createElement('div');
    divLogo.setAttribute('class', 'logoImg');
    if (thisClass.logoColumn) {
      divLogo.setAttribute('column', thisClass.logoColumn);
    }
    var aLogo = document.createElement('a');
    aLogo.setAttribute('href', thisClass.companyUrl);
    var imgLogo = document.createElement('img');
    imgLogo.setAttribute('src', './logo_small.svg');
    if (thisClass.logoWidth) {
      imgLogo.setAttribute('width', thisClass.logoWidth);
    }
    if (thisClass.logoHeight) {
      imgLogo.setAttribute('height', thisClass.logoHeight);
    }
    if (thisClass.logoAltText) {
      imgLogo.setAttribute('alt', thisClass.logoAltText);
    } else {
      imgLogo.setAttribute('alt', 'Logo image');
    }
    aLogo.appendChild(imgLogo);

    divLogo.appendChild(aLogo);
    //divLogo.appendChild(langswitch);
    var nav = document.createElement('nav');
    if (thisClass.navColumnSize) {
      nav.setAttribute('column', thisClass.navColumnSize);
    }
    nav.setAttribute('class', 'nav');
    var ul = document.createElement('ul');
    thisClass.topnav = ul;
    nav.appendChild(ul);
    div.appendChild(divLogo);
    div.appendChild(h1);
    div.appendChild(nav);
    thisClass.hideTopNavMenu();
    return div;
  }

  var createFooterRow = function() {
    var div = document.createElement('div');
    div.setAttribute('row', '');
    var p = document.createElement('p');
    var a = document.createElement('a');
    a.setAttribute('href', thisClass.companyUrl);
    a.appendChild(document.createTextNode(thisClass.companyName));
    var version = document.createTextNode('');
    thisClass.dataBindings.push(
      function() {
        if (thisClass.shortDeviceInfo) {
          version.nodeValue = " V" + thisClass.shortDeviceInfo.CurrentFirmwareVersion;
        }
      });
    p.appendChild(document.createTextNode('\u00A9 Copyright 2011-2020 '));
    p.appendChild(a);
    p.appendChild(version);

    div.appendChild(p);
    return div;
  }

  this.prepare = function() {
    if (thisClass.main) {
      return;
    }
    var body = document.body;
    if (thisClass.showExpandPageIcon) {
      var expandCollapse = document.createElement('span');
      var iconExpandCollapse = document.createTextNode('\ue215');
      expandCollapse.appendChild(iconExpandCollapse);
      expandCollapse.setAttribute('class', 'icon glyphicons cgrey expandCollapse');

      body.appendChild(expandCollapse);
    }
    
    // we use Concise containers and within the containers rows and columns
    // but first create the language row elements; this will also initialize the translations!
    var languageRow = createLanguageRow();

    if (thisClass.showHeaderMenu) {
      var headerMenuElem = document.createElement('header');
      headerMenuElem.setAttribute('container', '');
      headerMenuElem.setAttribute('class', 'siteHeaderMenu');
      body.appendChild(headerMenuElem);
      var headerMenuNav = document.createElement('nav');
      headerMenuNav.setAttribute('class', 'headerNav nav');
      thisClass.headerNav = document.createElement('ul');
      headerMenuNav.appendChild(thisClass.headerNav);
      headerMenuElem.appendChild(headerMenuNav);
    }
    // Page header
    var header = document.createElement('header');
    header.setAttribute('container', '');
    header.setAttribute('class', 'siteHeader');
    header.appendChild(createHeaderRow());
    if (thisClass.withHeaderSeparator) {
      var headerSeparator = document.createElement('hr');
      headerSeparator.setAttribute('class', 'headerSeparator');
      header.appendChild(headerSeparator);
    }
    body.appendChild(header);

    // Page content
    thisClass.main = document.createElement('main');
    thisClass.main.setAttribute('container', '');
    thisClass.main.setAttribute('class', 'siteContent');
    body.appendChild(thisClass.main);

    // Page footer
    var footer = document.createElement('footer');
    footer.setAttribute('container', '');
    footer.setAttribute('class', 'siteFooter');
    var sectionLeft = document.createElement('section');
    sectionLeft.setAttribute('class', 'left');
    var sectionRight = document.createElement('section');
    sectionRight.setAttribute('class', 'right');
    sectionLeft.appendChild(createFooterRow());
    sectionRight.appendChild(languageRow);
    footer.appendChild(sectionLeft);
    footer.appendChild(sectionRight);
    body.appendChild(footer);

    if (thisClass.showFooterMenu) {
      var footerMenuElem = document.createElement('footer');
      footerMenuElem.setAttribute('container', '');
      footerMenuElem.setAttribute('class', 'siteFooterMenu');
      body.appendChild(footerMenuElem);
      var footerMenuNav = document.createElement('nav');
      footerMenuNav.setAttribute('class', 'footerNav nav');
      thisClass.footerNav = document.createElement('ul');
      footerMenuNav.appendChild(thisClass.footerNav);
      footerMenuElem.appendChild(footerMenuNav);
    }

    thisClass.createNavMenu();

    if (thisClass.showExpandPageIcon) {
      if (typeof window.mainSite.localStorage !== 'undefined' && mainSite.localStorage.getItem('smallBody') == 'true'){
        body.setAttribute('class', 'smallBody');
        thisClass.topnav.parentElement.setAttribute('column', '12');
      }
      expandCollapse.onclick = function() {
        if (body.getAttribute('class') != 'smallBody'){
          if (typeof window.mainSite.localStorage !== 'undefined')
            mainSite.localStorage.setItem('smallBody', 'true');
          body.setAttribute('class', 'smallBody');
          thisClass.topnav.parentElement.setAttribute('column', '12');
          iconExpandCollapse.nodeValue = '\ue215';
        }
        else{
          if (typeof window.mainSite.localStorage !== 'undefined')
            mainSite.localStorage.setItem('smallBody', 'false');
          body.removeAttribute('class');
          thisClass.topnav.parentElement.setAttribute('column', '12');
          iconExpandCollapse.nodeValue = '\ue216';
        }
      }
    }
    
    // Page modals for messages
    thisClass.modal = document.createElement('div');
    thisClass.modal.setAttribute('id', 'modal');
    thisClass.modal.setAttribute('class', 'modal');
    body.appendChild(thisClass.modal);
    thisClass.modalWaitForContent = document.createElement('div');
    thisClass.modalWaitForContent.setAttribute('class', 'waitForContent modal');
    var spinner = document.createElement('div');
    spinner.setAttribute('class', 'spinner');
    thisClass.modalWaitForContent.appendChild(spinner);
    body.appendChild(thisClass.modalWaitForContent);
    //end createInitialStructure
    try {
      thisClass.setDeviceInfo(JSON.parse(mainSite.sessionStorage.getItem('deviceInfoCache')));
      thisClass.update();
    } catch (err) {}
    if (!thisClass.homepage) {
      thisClass.homepage = thisClass.pages['StatusPage'];
    }
    var target = location.hash;
    // bugfix if a user presses reload and the target is #modal
    if (target == "#modal") {
      target = "";
      // firefox needs a clean reload:
      location.hash = "";
      location.reload(true);
    }
    if (target.length > 1) {
      target = target.slice(1);
    }
    thisClass.requestData();
    IscWebService('getDeviceInfo', '', function(response) {
      thisClass.setDeviceInfo(response);
      if (response && response.data && response.data.DeviceName){
        thisClass.showTopNavMenu();
        thisClass.hideMenuElement('LoginPage');
        thisClass.show(target);
      }
      else {
        thisClass.show('LoginPage');
      }
    });
  };

  this.setDeviceInfo = function(response) {
    if (response && response.data) {
      if (Object.keys(response.data).length > 2) {
        thisClass.deviceInfo = response.data;
        try {
          mainSite.sessionStorage.setItem('deviceInfoCache', JSON.stringify(response));
        } catch (err) {}
      }
      thisClass.shortDeviceInfo = response.data;
    }
  }
  this.setCOs = function(response) {
    if (response && response.data) {
      thisClass.currentCOValues = response.data;
    }
  }

  this.setAppState = function(response) {
    if (response) {
      if (response.data) {
        thisClass.appState = response.data;
      } else if (response.error && response.id == '222') {
        thisClass.appState = null;
      } else {
        thisClass.appState = undefined;
      }
    } else {
      thisClass.appState = undefined;
    }
  }

  this.requestData = function() {
    if (!thisClass.updateProcess){
      thisClass.updateProcess = new UpdateProcess();
      thisClass.updateProcess.init();
    }
    if (!thisClass.timer.getDeviceInfo) {
      thisClass.timer.getDeviceInfo = new TimedRequest(INTERVAL.MINUTE, 'getDeviceInfo', '', thisClass.setDeviceInfo);
    } else {
      thisClass.timer.getDeviceInfo.timeout = INTERVAL.HALFMINUTE;
    }
    thisClass.timer.getDeviceInfo.addCallbackFunction(thisClass.update);
    thisClass.timer.getDeviceInfo.resetTimeout();
  }
  this.update = function() {
    //update data in site
    for (var i = 0; i < thisClass.dataBindings.length; i++) {
      thisClass.dataBindings[i]();
    }
  }
  this.timer = {};
}
/*
 * ##################################################### End MainSite #####################################################
 */
